@@ -28,15 +28,22 @@ from __future__ import absolute_import
 
 import re
 import copy
-import cStringIO
-from email.message import Message
-from email.generator import Generator
-from email.parser import Parser, FeedParser
+from io import BytesIO
+
+try:
+    from email.parser import BytesParser
+    from email.generator import BytesGenerator
+    from email.policy import SMTP
+except ImportError:
+    from email.parser import Parser
+    from email.generator import Generator
+
+from slimta.util import pycompat
 
 __all__ = ['Envelope']
 
-_HEADER_BOUNDARY = re.compile(r'\r?\n\s*?\n')
-_LINE_BREAK = re.compile(r'\r?\n')
+_HEADER_BOUNDARY = re.compile(br'\r?\n\s*?\n')
+_LINE_BREAK = re.compile(br'\r?\n')
 
 
 class Envelope(object):
@@ -48,6 +55,7 @@ class Envelope(object):
     :param headers: The message headers.
     :type headers: :class:`email.message.Message`
     :param message: String containing the message contents after the headers.
+    :type message: :py:obj:`bytes`
 
     """
 
@@ -63,7 +71,7 @@ class Envelope(object):
         #: message headers.
         self.headers = headers
 
-        #: String of message data, not including headers.
+        #: Bytestring of message data, not including headers.
         self.message = message
 
         #: Dictionary of information about the client that sent the message.
@@ -85,6 +93,31 @@ class Envelope(object):
         #: Timestamp when the message was received.
         self.timestamp = None
 
+    def _parse_data(self, data, *extra):
+        if pycompat.PY3:
+            return BytesParser(policy=SMTP).parse(BytesIO(data), *extra)
+        else:
+            return Parser().parse(BytesIO(data), *extra)
+
+    def _msg_generator(self, msg):
+        outfp = BytesIO()
+        if pycompat.PY3:
+            BytesGenerator(outfp, policy=SMTP).flatten(msg, False)
+            return outfp.getvalue()
+        else:
+            Generator(outfp).flatten(msg, False)
+            return re.sub(_LINE_BREAK, b'\r\n', outfp.getvalue())
+
+    def _merge_payloads(self, headers, payload):
+        if headers.get_payload():
+            new_msg = copy.deepcopy(headers)
+            headers.set_payload('')
+            for header in new_msg.keys():
+                del new_msg[header]
+            return self._msg_generator(new_msg).lstrip(b'\r\n') + payload
+        else:
+            return payload
+
     def prepend_header(self, name, value):
         """This method allows prepending a header to the message. The
         :attr:`.headers` object does not directly support header prepending
@@ -98,7 +131,7 @@ class Envelope(object):
         uses a deep-copying so internal datastructures are not shared.
 
         :param new_rcpts: If given, overwrite the :attr:`.recipients` list with
-                          this value inthe new object.
+                          this value in the new object.
         :returns: An exact, deep copy of the current object.
 
         """
@@ -110,34 +143,26 @@ class Envelope(object):
     def flatten(self):
         """Produces two strings representing the headers and message body.
 
-        :returns: Tuple of two strings: ``(header_data, message_data)``
+        :returns: Tuple of two bytestrings: ``(header_data, message_data)``
 
         """
-        outfp = cStringIO.StringIO()
-        Generator(outfp).flatten(self.headers, False)
-        header_data = re.sub(_LINE_BREAK, '\r\n', outfp.getvalue())
+        header_data = self._msg_generator(self.headers)
         return header_data, self.message
 
-    def _encode_parts(self, header_data, msg_data, encoder):
-        """Encodes any MIME part in the current message that is 8-bit."""
-        self.headers = None
-        self.message = None
-
-        parser = FeedParser()
-        parser.feed(header_data)
-        parser.feed(msg_data)
-        msg = parser.close()
+    def _encode_parts(self, encoder):
+        header_data, msg_data = self.flatten()
+        msg = self._parse_data(header_data + msg_data)
 
         for part in msg.walk():
             if not part.is_multipart():
                 payload = part.get_payload()
                 try:
                     payload.encode('ascii')
-                except UnicodeDecodeError:
+                except UnicodeError:
                     del part['Content-Transfer-Encoding']
                     encoder(part)
 
-        self.parse(msg)
+        self.parse_msg(msg)
 
     def encode_7bit(self, encoder=None):
         """.. versionadded:: 0.3.12
@@ -156,36 +181,41 @@ class Envelope(object):
         :raises: UnicodeDecodeError
 
         """
-        header_data, msg_data = self.flatten()
         try:
-            msg_data.encode('ascii')
+            self.message.decode('ascii')
         except UnicodeDecodeError:
             if not encoder:
                 raise
-            self._encode_parts(header_data, msg_data, encoder)
+            self._encode_parts(encoder)
 
-    def parse(self, data):
-        """Parses the given string or :class:`~email.message.Message` to
+    def parse_msg(self, msg):
+        """Parses the given :class:`~email.message.Message` to
         populate the :attr:`headers` and :attr:`message` attributes.
 
         :param data: The complete message, headers and message body.
-        :type data: :py:obj:`str` or :class:`~email.message.Message`
+        :type data: :class:`~email.message.Message`
+
+        """
+        self.parse(self._msg_generator(msg))
+
+    def parse(self, data):
+        """Parses the given string to populate the :attr:`headers` and
+        :attr:`message` attributes.
+
+        :param data: The complete message, headers and message body.
+        :type data: :py:obj:`bytes`
 
         """
-        if isinstance(data, Message):
-            outfp = cStringIO.StringIO()
-            Generator(outfp).flatten(data, False)
-            data = outfp.getvalue()
         match = re.search(_HEADER_BOUNDARY, data)
         if not match:
             header_data = data
-            payload = ''
+            payload = b''
         else:
             header_data = data[:match.end(0)]
             payload = data[match.end(0):]
-        self.headers = Parser().parsestr(header_data, True)
-        self.message = self.headers.get_payload() + payload
-        self.headers.set_payload('')
+
+        self.headers = self._parse_data(header_data, True)
+        self.message = self._merge_payloads(self.headers, payload)
 
     def __repr__(self):
         template = '<Envelope at {0}, sender={1!r}>'
