@@ -58,8 +58,8 @@ class SmtpRelayClient(RelayPoolClient):
     _client_class = Client
 
     def __init__(self, address, queue, socket_creator=None, ehlo_as=None,
-                 tls=None, tls_immediately=False,
-                 tls_required=False, tls_wrapper=None,
+                 context=None, auth_mechanism=None,
+                 tls_immediately=False, tls_required=False,
                  connect_timeout=10.0, command_timeout=10.0,
                  data_timeout=None, idle_timeout=None,
                  credentials=None, binary_encoder=None):
@@ -69,10 +69,10 @@ class SmtpRelayClient(RelayPoolClient):
         self.socket = None
         self.client = None
         self.ehlo_as = ehlo_as or hostname
-        self.tls = tls
+        self.context = context
+        self.auth_mechanism = auth_mechanism
         self.tls_immediately = tls_immediately
         self.tls_required = tls_required
-        self.tls_wrapper = tls_wrapper
         self.connect_timeout = connect_timeout
         self.command_timeout = command_timeout
         self.data_timeout = data_timeout or command_timeout
@@ -85,8 +85,7 @@ class SmtpRelayClient(RelayPoolClient):
         with Timeout(self.connect_timeout):
             self.socket = self.socket_creator(self.address)
         log.connect(self.socket, self.address)
-        self.client = self._client_class(self.socket, self.tls_wrapper,
-                                         self.address)
+        self.client = self._client_class(self.socket, self.address)
 
     @current_command(b'[BANNER]')
     def _banner(self):
@@ -104,12 +103,23 @@ class SmtpRelayClient(RelayPoolClient):
         with Timeout(self.command_timeout):
             ehlo = self.client.ehlo(ehlo_as)
         if ehlo.is_error():
+            if ehlo.code == '500':
+                return self._helo(ehlo_as)
             raise SmtpRelayError.factory(ehlo)
+        return ehlo
+
+    @current_command(b'HELO')
+    def _helo(self, ehlo_as):
+        with Timeout(self.command_timeout):
+            helo = self.client.helo(ehlo_as)
+        if helo.is_error():
+            raise SmtpRelayError.factory(helo)
+        return helo
 
     @current_command(b'STARTTLS')
     def _starttls(self):
         with Timeout(self.command_timeout):
-            starttls = self.client.starttls(self.tls)
+            starttls = self.client.starttls(self.context)
         if starttls.is_error() and self.tls_required:
             raise SmtpRelayError.factory(starttls)
 
@@ -120,16 +130,19 @@ class SmtpRelayClient(RelayPoolClient):
         except TypeError:
             credentials = self.credentials
         with Timeout(self.command_timeout):
-            auth = self.client.auth(*credentials)
+            auth = self.client.auth(*credentials,
+                                    mechanism=self.auth_mechanism)
         if auth.is_error():
             raise SmtpRelayError.factory(auth)
 
     def _handshake(self):
-        if self.tls and self.tls_immediately:
-            self.client.encrypt(self.tls)
-        self._banner()
-        self._ehlo()
-        if self.tls and not self.tls_immediately:
+        if self.tls_immediately:
+            self.client.encrypt(self.context)
+            self._banner()
+            self._ehlo()
+        else:
+            self._banner()
+            self._ehlo()
             if self.tls_required or 'STARTTLS' in self.client.extensions:
                 self._starttls()
                 self._ehlo()
@@ -144,7 +157,7 @@ class SmtpRelayClient(RelayPoolClient):
     @current_command(b'MAIL')
     def _mailfrom(self, sender):
         with Timeout(self.command_timeout):
-            mailfrom = self.client.mailfrom(sender)
+            mailfrom = self.client.mailfrom(sender, auth=False)
         if mailfrom and mailfrom.is_error():
             raise SmtpRelayError.factory(mailfrom)
         return mailfrom
